//*************************************************************************
//* ----------
//* CDCRawHist
//* ----------
//* (Description)
//*	This set of classes serves for CDC raw data analysis.
//*
//*	The following parameters can be specified in "jsf.conf". 
//*
//*	    CDCRawData.RFNFormat:       printf format to construct file name
//*	    CDCRawData.DoPlots:         (TRUE/FASLE)
//*         CDCRawHist.hPhii:           Phii 256.0, 0.0, 256.0
//*
//* (Update Record)
//*	1999/06/05  K.Fujii	Original version.
//*
//*************************************************************************

#include "JSFSteer.h"
#include "CDCRawHist.h"
#include <iostream.h>

ClassImp(CDCRawHist)

const Int_t CDCRawHist::kZoneX   = 4;
const Int_t CDCRawHist::kZoneY   = 3;
const Int_t CDCRawHist::kNh = kZoneX*kZoneY;

//___________________________________________________________________________
CDCRawHist::CDCRawHist(const Char_t *name, const Char_t *title)
	: JSFModule(name, title)
{
   hdefp = new THdef1[kNh];
   hdeft = new THdef1[kNh];
   Char_t st[256], std[256];
   Int_t i;
   for (i = 0; i < kNh; i++) {
      sprintf(st,"CDCRawHist.hPh%2.2d",i);
      sprintf(std,"Ph%2.2d, 256, 0.0, 256.0",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",hdefp[i].title,
	  		&hdefp[i].nbin, &hdefp[i].xmin, &hdefp[i].xmax);
      sprintf(st,"CDCRawHist.hTs%2.2d",i);
      sprintf(std,"Ts%2.2d, 2048, 0.0, 4096.0",i);
      sscanf(gJSF->Env()->GetValue(st,std),"%s %d,%g,%g",hdeft[i].title,
	  		&hdeft[i].nbin, &hdeft[i].xmin, &hdeft[i].xmax);
   }
   hPh = new TH1F* [kNh];
   hTs = new TH1F* [kNh];
}

//___________________________________________________________________________
CDCRawHist::~CDCRawHist()
{
   delete [] hdefp;
   delete [] hdeft;
   Int_t i;
   for (i = 0; i < kNh; i++) {
	delete hPh[i];
	delete hTs[i];
   }
   delete [] hPh; 
   delete [] hTs; 
}

//___________________________________________________________________________
Bool_t CDCRawHist::Initialize()
{
   TDirectory *last=gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < kNh; i++) {
      hPh[i] = new TH1F(hdefp[i].title,hdefp[i].title, hdefp[i].nbin,
    		        hdefp[i].xmin,hdefp[i].xmax);
      cout << i << " " << hdefp[i].title 
      		<< " n " << hdefp[i].nbin
      		<< " xmin " << hdefp[i].xmin
      		<< " xmax " << hdefp[i].xmax << endl;
      hTs[i] = new TH1F(hdeft[i].title,hdeft[i].title, hdeft[i].nbin,
    		        hdeft[i].xmin,hdeft[i].xmax);
      cout << i << " " << hdeft[i].title 
      		<< " n " << hdeft[i].nbin
      		<< " xmin " << hdeft[i].xmin
      		<< " xmax " << hdeft[i].xmax << endl;
   }
   hcp=new TCanvas("hcp","Pulse Height",20, 20, 200*kZoneX, 200*kZoneY);
   hcp->Divide(kZoneX,kZoneY);
   hct=new TCanvas("hct","Start Time",20, 20, 200*kZoneX, 200*kZoneY);
   hct->Divide(kZoneX,kZoneY);
   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t CDCRawHist::Process(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   CDCRawData    *raw = (CDCRawData*)gJSF->FindModule("CDCRawData");
   CDCRawDataBuf *evt = (CDCRawDataBuf *)raw->EventBuf();
   Int_t fmtid = evt->GetHeader()->GetFormatID();
   CDCBeginRunBuf *br = raw->GetBeginRunBuf();
   Int_t ncdc = evt->GetNADCData();
   cerr << " ncdc = " << ncdc << endl;
   TClonesArray *adcdata = evt->GetADCData();
   Int_t i;
   for (i = 0; i < ncdc; i++) {
      CDCRawADCDatum *adcdatum = (CDCRawADCDatum *)adcdata->UncheckedAt(i);
      Int_t a;
      switch (fmtid) {
         case 0: {
            a = adcdatum->GetChannelNo();
            break;
         }
         default: {
            Int_t id;
            Int_t cr = 0; 		// We have only 1 crate now.
            Int_t st = adcdatum->GetStationNo();
            Int_t ch = adcdatum->GetChannelNo();
	    if (ch > 1) continue;// TEMPORARY REMEDY FOR BROKEN FORMAT
            br->EChannelToDChannel(cr,st,ch,id,a);
            break;
         }
      }
      Int_t adc  = adcdatum->GetADC();
      Int_t tim  = adcdatum->GetTime();
      if ( a >= 0 && a < CDCRawHist::kNh ) {
		hPh[a]->Fill(adc);
		hTs[a]->Fill(tim);
      }
   }
   if (!(gROOT->IsBatch())) Draw();
   last->cd();
   return kTRUE;
}

//___________________________________________________________________________
void CDCRawHist::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < CDCRawHist::kNh; i++) {
      hcp->cd(i+1); hPh[i]->Draw(); hcp->Update();
      hct->cd(i+1); hTs[i]->Draw(); hct->Update();
   }
   last->cd();
}
